@@ -27,7 +27,7 @@
 
 (declare
   (run-time-macros)
-  (uses srfi-1 regex utils posix ports tcp match srfi-18 srfi-13)
+  (uses srfi-1 regex utils files posix ports tcp match srfi-18 srfi-13)
   (export move-file run:execute make/proc uninstall-extension
 	  install-extension install-program install-script setup-verbose-flag
 	  setup-install-flag installation-prefix chicken-prefix find-library
@@ -196,6 +196,7 @@
 (define *move-command* (if *windows-shell* 'move 'mv))
 (define *gzip-program* 'gzip)
 (define *tar-program* 'tar)
+
 (define *fetch-only* #f)
 (define *builddir-created* #f)
 (define *keep-stuff* #f)
@@ -419,7 +420,7 @@
 				 (any (lambda (dep)
 					  (let ((dep2 (fixmaketarget dep)))
 					    (unless (file-exists? dep2)
-					      (error (sprintf "dependancy ~a was not made~%" dep2)))
+					      (error (sprintf "dependency ~a was not made~%" dep2)))
 					    (and (> (file-modification-time dep2) date)
 						 dep2)) )
 					deps))))
@@ -1068,7 +1069,7 @@ EOF
 (define (fetch-file ext)
   (and (or *dont-ask*
 	   (yes-or-no?
-	    (sprintf "The extension ~A does not exist.~%Do you want to download it ?" ext)
+	    (sprintf "File ~A.egg or ~A.setup not found in the download directory. ~%Do you want to download .egg archive ?" ext ext)
 	    "yes") )
        (cond ((pathname-directory ext)
 	      (printf "Warning: no repository index available, trying direct download...~%" ext)
