@@ -26,7 +26,7 @@
 
 
 (declare
-  (uses srfi-1 match srfi-4 utils support compiler optimizer driver platform backend)
+  (uses srfi-1 match srfi-4 utils files support compiler optimizer driver platform backend)
   (run-time-macros) )
 
 
