@@ -27,7 +27,7 @@
 
 (declare
   (unit posix)
-  (uses scheduler regex extras utils)
+  (uses scheduler regex extras utils files)
   (disable-interrupts)
   (usual-integrations)
   (hide ##sys#stat group-member _get-groups _ensure-groups posix-error
