@@ -1,6 +1,7 @@
 ;;;; support.scm - Miscellaneous support code for the CHICKEN compiler
 ;
-; Copyright (c) 2000-2006, Felix L. Winkelmann
+; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2008, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -22,43 +23,35 @@
 ; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 ; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 ; POSSIBILITY OF SUCH DAMAGE.
-;
-; Send bugs, suggestions and ideas to: 
-;
-; felix@call-with-current-continuation.org
-;
-; Felix L. Winkelmann
-; Unter den Gleichen 1
-; 37130 Gleichen
-; Germany
 
 
 (declare (unit support))
 
 
-#{compiler
+(private compiler
   compiler-arguments process-command-line dump-nodes dump-undefined-globals
   default-standard-bindings default-extended-bindings side-effecting-standard-bindings
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home optimization-iterations compiler-cleanup-hook decompose-lambda-list
-  foreign-type-table-size file-io-only banner custom-declare-alist parse-easy-ffi disabled-warnings
+  file-io-only banner custom-declare-alist disabled-warnings internal-bindings
   unit-name insert-timer-checks used-units source-filename pending-canonicalizations
-  foreign-declarations block-compilation analysis-database-size line-number-database-size
+  foreign-declarations block-compilation line-number-database-size
   target-heap-size target-stack-size check-global-exports check-global-imports
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
-  dependency-list broken-constant-nodes inline-substitutions-enabled no-c-syntax-checks
-  always-bound-to-procedure block-variable-literal copy-node! valid-c-identifier? tree-copy copy-node-tree-and-rename
-  direct-call-ids foreign-type-table first-analysis scan-sharp-greater-string enable-sharp-greater-read-syntax
+  dependency-list broken-constant-nodes inline-substitutions-enabled emit-syntax-trace-info
+  always-bound-to-procedure block-variable-literal? copy-node! valid-c-identifier? tree-copy copy-node-tree-and-rename
+  direct-call-ids foreign-type-table first-analysis scan-sharp-greater-string
+  make-block-variable-literal block-variable-literal-name
   expand-profile-lambda profile-lambda-list profile-lambda-index profile-info-vector-name
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database scan-toplevel-assignments
   perform-cps-conversion analyze-expression simplifications perform-high-level-optimizations perform-pre-optimization!
-  reorganize-recursive-bindings substitution-table simplify-named-call check-c-syntax
+  reorganize-recursive-bindings substitution-table simplify-named-call
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
-  transform-direct-lambdas! finish-foreign-result compressable-literal csc-control-file
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  transform-direct-lambdas! finish-foreign-result csc-control-file
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words words->bytes check-and-open-input-file close-checked-input-file fold-inner
   constant? basic-literal? source-info->string import-table
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
@@ -74,11 +67,11 @@
   default-optimization-iterations chop-separator chop-extension follow-without-loop dump-exported-globals
   generate-code make-variable-list make-argument-list generate-foreign-stubs foreign-type-declaration
   foreign-argument-conversion foreign-result-conversion final-foreign-type debugging export-list block-globals
-  lookup-exports-file constant-declarations
-  make-random-name foreign-type-convert-result foreign-type-convert-argument process-custom-declaration}
+  lookup-exports-file constant-declarations process-lambda-documentation big-fixnum?
+  compiler-macro-table register-compiler-macro export-dump-hook export-import-hook
+  make-random-name foreign-type-convert-result foreign-type-convert-argument process-custom-declaration)
 
 
-(include "parameters")
 (include "tweaks")
 (include "banner")
 
@@ -122,10 +115,16 @@
 (set! ##sys#syntax-error-hook
   (lambda (msg . args)
     (let ([out (current-error-port)])
-      (fprintf out "Syntax error: ~a~%" msg) 
-      (for-each (lambda (x) (write x out) (newline out)) args)
+      (fprintf out "Syntax error: ~a~%~%" msg) 
+      (for-each (cut fprintf out "\t~s~%" <>) args)
+      (print-call-chain out 0 ##sys#current-thread "\n\tExpansion history:\n")
       (exit 70) ) ) )
 
+(set! syntax-error ##sys#syntax-error-hook)
+
+(define (emit-syntax-trace-info info cntr) 
+  (##core#inline "C_emit_syntax_trace_info" info cntr ##sys#current-thread) )
+
 (define (map-llist proc llist)
   (let loop ([llist llist])
     (cond [(null? llist) '()]
@@ -162,9 +161,6 @@
 	((string? x) (string->symbol x))
 	(else (string->symbol (sprintf "~a" x))) ) )
 
-(define (flonum? x)
-  (and (number? x) (not (exact? x))) )
-
 (define (build-lambda-list vars argc rest)
   (let loop ((vars vars) (n argc))
     (cond ((or (zero? n) (null? vars)) (or rest '()))
@@ -181,7 +177,7 @@
 	  '(#\")
 	  (let* ((c (car chars))
 		 (code (char->integer c)) )
-	    (if (or (< code 32) (> code 128) (memq c '(#\" #\' #\\ #\?)))
+	    (if (or (< code 32) (>= code 127) (memq c '(#\" #\' #\\ #\?)))
 		(append '(#\\)
 			(cond ((< code 8) '(#\0 #\0))
 			      ((< code 64) '(#\0))
@@ -253,25 +249,13 @@
       (symbol? x) ) )
 
 (define (immediate? x)
-  (or (fixnum? x)
+  (or (and (fixnum? x) (not (big-fixnum? x))) ; 64-bit fixnums would result in platform-dependent .c files
       (eq? (##core#undefined) x)
       (null? x)
       (eof-object? x)
       (char? x)
       (boolean? x) ) )
 
-(define (compressable-literal lit t)
-  (let* ([count 0]
-	 [f (let rec ([x lit])
-	      (set! count (add1 count))
-	      (cond [(or (number? x) (char? x) (string? x) (boolean? x) (null? x) (symbol? x))] ; 1
-		    [(pair? x)		; car + cdr
-		     (set! count (sub1 count))
-		     (and (rec (car x)) (rec (cdr x))) ]
-		    [(vector? x) (every rec (vector->list x))] ; 1 + elements
-		    [else #f] ) ) ] )
-    (and f (> count t) count) ) )
-
 (define (basic-literal? x)
   (or (null? x)
       (symbol? x)
@@ -339,6 +323,9 @@
 
 (define decompose-lambda-list ##sys#decompose-lambda-list)
 
+(define (process-lambda-documentation id doc proc)
+  proc)					; Hook this
+
 
 ;;; Profiling instrumentation:
 
@@ -444,45 +431,53 @@
   (let ((names '((captured . cpt) (assigned . set) (boxed . box) (global . glo) (assigned-locally . stl)
 		 (contractable . con) (standard-binding . stb) (foldable . fld) (simple . sim) (inlinable . inl)
 		 (side-effecting . sef) (collapsable . col) (removable . rem) (constant . con)
-		 (undefined . und) (replacing . rpg) (unused . uud) (extended-binding . xtb)
-		 (customizable . cst) (has-unused-parameters . hup) (boxed-rest . bxr) ) ) )
+		 (undefined . und) (replacing . rpg) (unused . uud) (extended-binding . xtb) (inline-export . ilx)
+		 (customizable . cst) (has-unused-parameters . hup) (boxed-rest . bxr) ) ) 
+	(omit #f))
     (lambda (db)
+      (unless omit
+	(set! omit 
+	  (append default-standard-bindings
+		  default-extended-bindings
+		  internal-bindings) ) )
       (##sys#hash-table-for-each
        (lambda (sym plist)
 	 (let ([val #f]
 	       [pval #f]
 	       [csites '()]
 	       [refs '()] )
-	   (write sym)
-	   (let loop ((es plist))
-	     (if (pair? es)
-		 (begin
-		   (case (caar es)
-		     ((captured assigned boxed global contractable standard-binding foldable assigned-locally
-		       side-effecting collapsable removable undefined replacing unused simple inlinable
-		       has-unused-parameters extended-binding customizable constant boxed-rest)
-		      (printf "\t~a" (cdr (assq (caar es) names))) )
-		     ((unknown)
-		      (set! val 'unknown) )
-		     ((value)
-		      (unless (eq? val 'unknown) (set! val (cdar es))) )
-		     ((potential-value)
-		      (set! pval (cdar es)) )
-		     ((replacable home contains contained-in use-expr closure-size rest-parameter
-		       o-r/access-count captured-variables explicit-rest)
-		      (printf "\t~a=~s" (caar es) (cdar es)) )
-		     ((references)
-		      (set! refs (cdar es)) )
-		     ((call-sites)
-		      (set! csites (cdar es)) )
-		     (else (bomb "Illegal property" (car es))) )
-		   (loop (cdr es)) ) ) )
-	   (cond [(and val (not (eq? val 'unknown)))
-		  (printf "\tval=~s" (cons (node-class val) (node-parameters val))) ]
-		 [pval (printf "\tpval=~s" (cons (node-class val) (node-parameters val)))] )
-	   (when (pair? refs) (printf "\trefs=~s" (length refs)))
-	   (when (pair? csites) (printf "\tcss=~s" (length csites)))
-	   (newline) ) )
+	   (unless (memq sym omit)
+	     (write sym)
+	     (let loop ((es plist))
+	       (if (pair? es)
+		   (begin
+		     (case (caar es)
+		       ((captured assigned boxed global contractable standard-binding foldable assigned-locally
+				  side-effecting collapsable removable undefined replacing unused simple inlinable inline-export
+				  has-unused-parameters extended-binding customizable constant boxed-rest)
+			(printf "\t~a" (cdr (assq (caar es) names))) )
+		       ((unknown)
+			(set! val 'unknown) )
+		       ((value)
+			(unless (eq? val 'unknown) (set! val (cdar es))) )
+		       ((potential-value)
+			(set! pval (cdar es)) )
+		       ((replacable home contains contained-in use-expr closure-size rest-parameter
+				    o-r/access-count captured-variables explicit-rest)
+			(printf "\t~a=~s" (caar es) (cdar es)) )
+		       ((references)
+			(set! refs (cdar es)) )
+		       ((call-sites)
+			(set! csites (cdar es)) )
+		       (else (bomb "Illegal property" (car es))) )
+		     (loop (cdr es)) ) ) )
+	     (cond [(and val (not (eq? val 'unknown)))
+		    (printf "\tval=~s" (cons (node-class val) (node-parameters val))) ]
+		   [(and pval (not (eq? pval 'unknown)))
+		    (printf "\tpval=~s" (cons (node-class pval) (node-parameters pval)))] )
+	     (when (pair? refs) (printf "\trefs=~s" (length refs)))
+	     (when (pair? csites) (printf "\tcss=~s" (length csites)))
+	     (newline) ) ) )
        db) ) ) )       
 
 
@@ -505,7 +500,7 @@
   (let ([count 0])
     (define (walk x)
       (cond ((symbol? x) (varnode x))
-	    ((not-pair? x) (bomb "bad expression"))
+	    ((not-pair? x) (bomb "bad expression" x))
 	    ((symbol? (car x))
 	     (case (car x)
 	       ((##core#global-ref) (make-node '##core#global-ref (list (cadr x)) '()))
@@ -536,11 +531,14 @@
 		   (car x)
 		   (list (if (and (pair? arg) (eq? 'quote (car arg))) (cadr arg) arg))
 		   (map walk (cddr x)) ) ) )
-	       ((set! ##core#inline ##core#callunit) 
+	       ((##core#inline ##core#callunit) 
 		(make-node (car x) (list (cadr x)) (map walk (cddr x))) )
 	       ((##core#proc)
 		(make-node '##core#proc (list (cadr x) #t) '()) )
-	       ((##core#set!) (make-node 'set! (list (cadr x)) (map walk (cddr x))))
+	       ((set! ##core#set!)
+		(make-node
+		 'set! (list (cadr x))
+		 (map walk (cddr x))))
 	       ((##core#foreign-callback-wrapper)
 		(let ([name (cadr (second x))])
 		  (make-node
@@ -553,8 +551,7 @@
 	       ((##core#app)
 		(make-node '##core#call '(#t) (map walk (cdr x))) )
 	       (else
-		(receive
-		    (name ln) (get-line-2 x)
+		(receive (name ln) (get-line-2 x)
 		  (make-node
 		   '##core#call
 		   (list (cond [(memq name always-bound-to-procedure)
@@ -752,19 +749,29 @@
 
 ;;; Some safety checks and database dumping:
 
+(define (export-dump-hook db file) (void))
+
 (define (dump-exported-globals db file)
   (unless block-compilation
     (with-output-to-file file
       (lambda ()
-	(##sys#hash-table-for-each
-	 (lambda (sym plist)
-	   (when (and (assq 'global plist) 
-		      (assq 'assigned plist)
-		      (or (and export-list (memq sym export-list))
-			  (not (memq sym block-globals)) ) )
-	     (write sym) 
-	     (newline) ) )
-	 db) ) ) ) )
+	(let ((exports '()))
+	  (##sys#hash-table-for-each
+	   (lambda (sym plist)
+	     (when (and (assq 'global plist) 
+			(assq 'assigned plist)
+			(or (and export-list (memq sym export-list))
+			    (not (memq sym block-globals)) ) )
+	       (set! exports (cons sym exports)) ) )
+	   db)
+	  (for-each 
+	   (lambda (s)
+	     (write s)
+	     (newline) )
+	   (sort exports
+		 (lambda (s1 s2)
+		   (string<? (##sys#slot s1 1) (##sys#slot s2 1)))) )
+	  (export-dump-hook db file) ) ) ) ) )
 
 (define (dump-undefined-globals db)
   (##sys#hash-table-for-each
@@ -800,6 +807,8 @@
 		(compiler-warning 'var "variable `~s' used but not imported" sym) ) ) ) ) )
    db) )
 
+(define (export-import-hook x id) (void))
+
 (define (lookup-exports-file id)
   (and-let* ((xfile (##sys#resolve-include-filename 
 		     (string-append (->string id) ".exports")
@@ -807,7 +816,12 @@
 	     ((file-exists? xfile)) )
     (when verbose-mode 
       (printf "loading exports file ~a ...~%" xfile) )
-    (for-each (cut ##sys#hash-table-set! import-table <> id) (read-file xfile)) ) )
+    (for-each
+     (lambda (exp)
+       (if (symbol? exp)
+	   (##sys#hash-table-set! import-table exp id) 
+	   (export-import-hook exp id) ) )
+     (read-file xfile)) ) )
 
 
 ;;; Compute general statistics from analysis database:
@@ -894,7 +908,7 @@
 	     [(int unsigned-int short unsigned-short byte unsigned-byte int32 unsigned-int32)
 	      (if unsafe param `(##sys#foreign-fixnum-argument ,param))]
 	     [(float double number) (if unsafe param `(##sys#foreign-flonum-argument ,param))]
-	     [(pointer byte-vector scheme-pointer) ; pointer is DEPRECATED
+	     [(pointer byte-vector blob scheme-pointer) ; pointer and byte-vector are DEPRECATED
 	      (let ([tmp (gensym)])
 		`(let ([,tmp ,param])
 		   (if ,tmp
@@ -902,7 +916,7 @@
 			    tmp
 			    `(##sys#foreign-block-argument ,tmp) )
 		       '#f) ) ) ]
-	     [(nonnull-pointer nonnull-scheme-pointer nonnull-byte-vector) ; nonnull-pointer is DEPRECATED
+	     [(nonnull-pointer nonnull-scheme-pointer nonnull-blob nonnull-byte-vector) ; nonnull-pointer and nonnull-byte-vector are DEPRECATED
 	      (if unsafe
 		  param
 		  `(##sys#foreign-block-argument ,param) ) ]
@@ -926,7 +940,7 @@
 	      (if unsafe
 		  param
 		  `(##sys#foreign-unsigned-integer-argument ,param) ) ]
-	     [(c-pointer)
+	     [(c-pointer c-string-list c-string-list*)
 	      (let ([tmp (gensym)])
 		`(let ([,tmp ,param])
 		   (if ,tmp
@@ -934,7 +948,7 @@
 		       '#f) ) ) ]
 	     [(nonnull-c-pointer)
 	      `(##sys#foreign-pointer-argument ,param) ]
-	     [(c-string c-string*)
+	     [(c-string c-string* unsigned-c-string*)
 	      (let ([tmp (gensym)])
 		`(let ([,tmp ,param])
 		   (if ,tmp
@@ -942,7 +956,7 @@
 			    `(##sys#make-c-string ,tmp)
 			    `(##sys#make-c-string (##sys#foreign-string-argument ,tmp)) )
 		       '#f) ) ) ]
-	     [(nonnull-c-string nonnull-c-string*)
+	     [(nonnull-c-string nonnull-c-string* nonnull-unsigned-c-string*)
 	      (if unsafe 
 		  `(##sys#make-c-string ,param)
 		  `(##sys#make-c-string (##sys#foreign-string-argument ,param)) ) ]
@@ -971,6 +985,8 @@
 		       [('nonnull-instance . _)
 			`(slot-ref ,param 'this) ]
 		       [('const t) (repeat t)]
+		       [('enum _) 
+			(if unsafe param `(##sys#foreign-integer-argument ,param))]
 		       [((or 'nonnull-pointer 'nonnull-c-pointer) . _)
 			`(##sys#foreign-pointer-argument ,param) ]
 		       [_ param] ) ]
@@ -1015,7 +1031,9 @@
        ((char int short bool void unsigned-short scheme-object unsigned-char unsigned-int byte unsigned-byte
 	      int32 unsigned-int32) 
 	0)
-       ((c-string nonnull-c-string c-pointer nonnull-c-pointer symbol c-string* nonnull-c-string*)
+       ((c-string nonnull-c-string c-pointer nonnull-c-pointer symbol c-string* nonnull-c-string*
+                  unsigned-c-string* nonnull-unsigned-c-string*
+		  c-string-list c-string-list*)
 	(words->bytes 3) )
        ((unsigned-integer long integer unsigned-long integer32 unsigned-integer32)
 	(words->bytes 4) )
@@ -1043,7 +1061,8 @@
        ((char int short bool unsigned-short unsigned-char unsigned-int long unsigned-long byte unsigned-byte
 	      c-pointer pointer nonnull-c-pointer unsigned-integer integer float c-string symbol
 	      scheme-pointer nonnull-scheme-pointer int32 unsigned-int32 integer32 unsigned-integer32
-	      nonnull-c-string c-string* nonnull-c-string*) ; pointer and nonnull-pointer are DEPRECATED
+              unsigned-c-string* nonnull-unsigned-c-string*
+	      nonnull-c-string c-string* nonnull-c-string* c-string-list c-string-list*) ; pointer and nonnull-pointer are DEPRECATED
 	(words->bytes 1) )
        ((double number)
 	(words->bytes 2) )
@@ -1065,13 +1084,15 @@
   (case type
     [(c-string) `(##sys#peek-c-string ,body '0)]
     [(nonnull-c-string) `(##sys#peek-nonnull-c-string ,body '0)]
-    [(c-string*) `(##sys#peek-and-free-c-string ,body '0)]
-    [(nonnull-c-string*) `(##sys#peek-and-free-nonnull-c-string ,body '0)]
+    [(c-string* unsigned-c-string*) `(##sys#peek-and-free-c-string ,body '0)]
+    [(nonnull-c-string* nonnull-unsigned-c-string*) `(##sys#peek-and-free-nonnull-c-string ,body '0)]
     [(symbol) `(##sys#intern-symbol (##sys#peek-c-string ,body '0))]
+    [(c-string-list) `(##sys#peek-c-string-list ,body '#f)]
+    [(c-string-list*) `(##sys#peek-and-free-c-string-list ,body '#f)]
     [else
      (match type
        [((or 'instance 'instance-ref) cname sname)
-	`(##tinyclos#make-instance-from-pointer ,body ,sname) ]
+	`(##tinyclos#make-instance-from-pointer ,body ,sname) ] ;XXX eggified, needs better treatment...
        [('nonnull-instance cname sname)
 	`(make ,sname 'this ,body) ]
        [_ body] ) ] ) )
@@ -1180,7 +1201,8 @@
 
 (define (chop-separator str)
   (let ([len (sub1 (string-length str))])
-    (if (and (> len 0) (char=? (string-ref str len) ##sys#pathname-directory-separator))
+    (if (and (> len 0) 
+	     (memq (string-ref str len) '(#\\ #\/)))
 	(substring str 0 len)
 	str) ) )
 
@@ -1194,9 +1216,9 @@
 
 ;;; Print version/usage information:
 
-(define (print-version . b)
-  (when (:optional b #f) (printf "~A" banner))
-  (printf "~A~%~A" (chicken-version #t) copyright) )
+(define (print-version #!optional b)
+  (when b (print* +banner+))
+  (print (chicken-version #t)) )
 
 (define (print-usage)
   (print-version)
@@ -1224,11 +1246,6 @@ Usage: chicken FILENAME OPTION ...
   Language options:
 
     -feature SYMBOL             register feature identifier
-    -ffi-define SYMBOL          define preprocessor macro for ``easy'' FFI parser
-    -ffi-include-path PATH      set include path for ``easy'' FFI parser
-    -ffi-no-include             don't parse include files when encountered by the FFI parser
-    -ffi                        parse and compile C/C++ code and generate Scheme bindings
-    -ffi-parse                  parse C/C++ code without including it
 
   Syntax related options:
 
@@ -1293,8 +1310,6 @@ Usage: chicken FILENAME OPTION ...
   Obscure options:
 
     -debug MODES                display debugging output for the given modes
-    -compress-literals NUMBER   compile literals above threshold as strings
-    -disable-c-syntax-checks    disable syntax check of C code fragments
     -unsafe-libraries           marks the generated file as being linked
                                 with the unsafe runtime system
     -raw                        do not generate implicit init- and exit code			       
@@ -1360,7 +1375,7 @@ EOF
 (define (display-real-name-table)
   (##sys#hash-table-for-each
    (lambda (key val)
-     (printf "~S ~S~%" key val) )
+     (printf "~S\t~S~%" key val) )
    real-name-table) )
 
 (define (source-info->string info)
@@ -1400,55 +1415,14 @@ EOF
 
 ;;; "#> ... <#" syntax:
 
-(define (enable-sharp-greater-read-syntax)
-  (set! ##sys#user-read-hook
-    (let ([old-hook ##sys#user-read-hook])
-      (lambda (char port)
-	(if (char=? #\> char)	       
-	    (let ([_ (read-char port)]		; swallow #\>
-		  [decl #f]
-		  [parse #f]
-		  [exec #f] )
-	      (case (peek-char port)
-		[(#\!)
-		 (read-char port)
-		 (set! parse #t)
-		 (set! decl #t) ]
-		[(#\?)
-		 (read-char port)
-		 (set! parse #t) ]
-		[(#\:)
-		 (read-char port)
-		 (set! exec #t) ]
-		[(#\()
-		 (let ([head (read port)])
-		   (match head
-		     [_ (for-each
-			 (lambda (t)
-			   (case t
-			     [(declare) (set! decl #t)]
-			     [(parse) (set! parse #t)]
-			     [(execute) (set! exec #t)]
-			     [else (quit "invalid tag in `#>(...) ...<#' form" t)] ) )
-			 head) ] ) ) ]
-		[else (set! decl #t)] )
-	      (let ([text (scan-sharp-greater-string port)])
-		(check-c-syntax text)
-		`(begin
-		   ,@(if decl
-			 `((declare (foreign-declare ,text)))
-			 '() )
-		   ,@(if parse
-			 `((declare (foreign-parse ,text)))
-			 '() )
-		   ,@(if exec
-			 (let ([tmp (gensym 'code_)])
-			   `((declare 
-			       (foreign-declare
-				,(sprintf "static C_word ~A() { ~A; return C_SCHEME_UNDEFINED; }\n" tmp text) ) )
-			     (##core#inline ,(symbol->string tmp)) ) )
-			 '() ) ) ) )
-	    (old-hook char port) ) ) ) ) )
+(set! ##sys#user-read-hook
+  (let ([old-hook ##sys#user-read-hook])
+    (lambda (char port)
+      (if (char=? #\> char)	       
+	  (let* ((_ (read-char port))		; swallow #\>
+		 (text (scan-sharp-greater-string port)))
+	    `(declare (foreign-declare ,text)) )
+	  (old-hook char port) ) ) ) )
 
 (define (scan-sharp-greater-string port)
   (let ([out (open-output-string)])
@@ -1470,8 +1444,6 @@ EOF
 	       (write-char c out)
 	       (loop) ] ) ) ) ) )
 
-(enable-sharp-greater-read-syntax)
-
 
 ;;; Custom declarations:
 
@@ -1504,3 +1476,36 @@ EOF
 	  (close-output-port csc-control-file)
 	  (old) ) ) ) )
   (fprintf csc-control-file "~S~%" item) )
+
+
+;;; Compiler macro registration
+
+(define (register-compiler-macro name llist body)
+  (unless compiler-macro-table
+    (set! compiler-macro-table (make-vector 301 '())) )
+  (call/cc
+   (lambda (return)
+     (let* ((wvar (gensym))
+	    (llist
+	     (let loop ((llist llist))
+	       (cond ((not (pair? llist)) llist)
+		     ((eq? #:whole (car llist))
+		      (unless (pair? (cdr llist))
+			(return #f) )
+		      (set! wvar (cadr llist))
+		      (cddr llist) )
+		     (else (cons (car llist) (loop (cdr llist)))) ) ) ) )
+       (##sys#hash-table-set!
+	compiler-macro-table
+	name
+	(eval `(lambda (,wvar) (apply (lambda ,llist ,@body) (cdr ,wvar))) ) )
+       #t) ) ) )
+
+
+;;; 64-bit fixnum?
+
+(define (big-fixnum? x)
+  (and (fixnum? x)
+       (##sys#fudge 3)			; 64 bit?
+       (or (fx> x 1073741823)
+	   (fx< x -1073741824) ) ) )
